/*-
 * Copyright 2011 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef __CX88_DEBUG_H__
#define __CX88_DEBUG_H__

#include <iostream>
#include <time.h>
#include "cx88_xml_config.h"

#ifdef _DIAGNOSTIC
#define DIAGNOSTIC(stmt) stmt;
#else
#define DIAGNOSTIC(stmt)
#endif

extern std::ostream& cx88_debug_timestamp(std::ostream &stream);

#define CX88_DEBUG_COLOR_DEFAULT "\x1b[0m"
#define CX88_DEBUG_COLOR_RED     "\x1b[31m"
#define CX88_DEBUG_COLOR_GREEN   "\x1b[32m"
#define CX88_DEBUG_COLOR_YELLOW  "\x1b[33m"
#define CX88_DEBUG_COLOR_BLUE    "\x1b[34m"
#define CX88_DEBUG_COLOR_MAGENTA "\x1b[35m"
#define CX88_DEBUG_COLOR_CYAN    "\x1b[36m"

class cx88_colorizer
{
   public:
      cx88_colorizer(int fd, const char *color);
      ~cx88_colorizer();
   private:
      int m_fd;
};

extern const char *cx88_debug_color(const char *color);

#define CX88_DEBUG_NORM cx88_debug_color(CX88_DEBUG_COLOR_CYAN)
#define CX88_DEBUG_WARN cx88_debug_color(CX88_DEBUG_COLOR_YELLOW)
#define CX88_DEBUG_ERR cx88_debug_color(CX88_DEBUG_COLOR_RED)
#define CX88ERR (cx88_debug_timestamp(std::cerr << CX88_DEBUG_ERR))
#define CX88LOG (cx88_debug_timestamp(std::clog << CX88_DEBUG_NORM))
#define CX88WARN (cx88_debug_timestamp(std::clog << CX88_DEBUG_WARN))
#define CX88OUT std::cout << cx88_debug_color(CX88_DEBUG_COLOR_CYAN)
#define CX88_DEBUG_H1 cx88_debug_color(CX88_DEBUG_COLOR_CYAN)
#define CX88_DEBUG_H2 cx88_debug_color(CX88_DEBUG_COLOR_MAGENTA)
#define CX88_DEBUG_H3 cx88_debug_color(CX88_DEBUG_COLOR_BLUE)
#define CX88_DEBUG_TABLE(prefix, key, val) \
   prefix << CX88_DEBUG_H1 << setw(30 - strlen(prefix)) << setfill(' ') << left << key << setw(0) << CX88_DEBUG_H2 << val << CX88_DEBUG_H1 << std::endl

extern void cx88_list_devices(cx88_xml_config &xmlconf, const char *header, const char *pattern);

#endif

